// Injection Wizard
// --An application for injecting traffic in WEP protected Wi-Fi networks--
// Version: 1.0
// Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package cliente;

import java.awt.event.*;
import javax.swing.JOptionPane;
import java.io.IOException;
import java.util.ResourceBundle;
import java.util.Timer;
import java.util.TimerTask;


public class AcceptSt1Listener implements ActionListener, MsgSubscriber{

    private MainWindow window=null;    
    private BSSInfo selectbss=null;
    private volatile boolean done=false;
    private ReportUser reporter=null;

    private class ReportUser implements Runnable{
        private int type=0;
        private String usermsg="...";
        public void report(int type, String usermsg){
            this.type=type;
            this.usermsg=usermsg;
        }
        public void run(){
            window.reportUser(type, usermsg);
        }
    }

    private class CancelButtonTask extends TimerTask implements Runnable{
        private boolean first=true;

        public void run(){
            if(first){
                first=false;
                MainWindow.updateWindow(this);
            }else{
                if(!done)
                    window.enableCancel(true);}
        }
    }

    public AcceptSt1Listener(MainWindow mainwin){
        window=mainwin;
        reporter=new ReportUser();
    }

    public void actionPerformed(ActionEvent evt){
        CommClient commcli=window.getCommClient();
        ResourceBundle bundle=window.getBundle();
        String[] opts={bundle.getString("LABELYES"),
                       bundle.getString("LABELNO")};
        String err=bundle.getString("ERR"), msgerr=null;
        String ans=bundle.getString("ANS"), msgans=null;
        Timer timer=null;
        TimerTask timtask=null;
        boolean msgsend=false;
        byte[] request=null, response=null;
        int opt=JOptionPane.NO_OPTION;

        try{            
            selectbss=window.getSelectedBSS();
            if(selectbss==null){
                msgerr=bundle.getString("ERR_NOBSS");
                JOptionPane.showMessageDialog(window, msgerr, err,
                                              JOptionPane.ERROR_MESSAGE);
                opt=JOptionPane.CANCEL_OPTION;
            }else if(selectbss.getEssid()==null){
                msgans=bundle.getString("INF_NETID");
                opt=JOptionPane.showOptionDialog(window, msgans, ans,
                                                 JOptionPane.YES_NO_OPTION,
                                                 JOptionPane.QUESTION_MESSAGE,
                                                 null, opts, opts[0]);
                if(opt==JOptionPane.NO_OPTION)
                    opt=JOptionPane.CANCEL_OPTION;
            }

            if(opt!=JOptionPane.CANCEL_OPTION){
                request=new byte[8];
                request[0]=Msg.JOIN;
                byte[] rawmac=MsgOptParser.getRawMAC(selectbss.getBssid());
                System.arraycopy(rawmac, 0, request, 1, 6);

                if(opt==JOptionPane.YES_OPTION){
                    request[7]=0;
                    msgans=bundle.getString("INF_SRCH");
                    window.reportUser(MainWindow.TYPEINFO, msgans);
                    timer=new Timer("Enable Cancel Button");
                    timtask=new CancelButtonTask();
                }else
                    request[7]=1;

                commcli.sendMsg(request);
                msgsend=true;
                response=commcli.getResponse();
                if(response[1]==Msg.DOING){
                    window.enableAccept(false);
                    window.enableScan(false);
                    if(timer!=null)
                        timer.schedule(timtask, 3000);
                }else
                    throw new IllegalStateException("Error code: "+response[2]);
            }
            
        }catch(IOException io){
            io.printStackTrace(System.err);
            err=bundle.getString("ERR");
            msgerr=null;
            if(msgsend)
                msgerr=bundle.getString("ERR_NORESPONSE");
            else
                msgerr=bundle.getString("ERR_BREAKCONNECT");

            JOptionPane.showMessageDialog(window, msgerr, err,
                                          JOptionPane.ERROR_MESSAGE);

        }catch(IllegalStateException ile){
            ile.printStackTrace(System.err);
            err=bundle.getString("ERR");
            msgerr=bundle.getString("ERR_SERVER");
            JOptionPane.showMessageDialog(window, msgerr, err,
                                          JOptionPane.ERROR_MESSAGE);
        }
    }

    public void deliverMsg(byte[] msg){
        ResourceBundle bdle=window.getBundle();
        StringBuffer sb=new StringBuffer();

        if(msg[2]==Msg.DONE){
            done=true;
            sb.append(bdle.getString("INF_JOIN"));
            sb.append(' ');
            sb.append(selectbss.getBssid());
            final String usermsg=sb.toString();
            reporter.report(MainWindow.TYPEINFO, usermsg);
            MainWindow.updateWindow(reporter);
            try{
                nextStage();
            }catch(IllegalStateException ile){
                String err=bdle.getString("ERR");
                String msgerr=bdle.getString("ERR_SERVER");
                JOptionPane.showMessageDialog(window, msgerr, err,
                                              JOptionPane.ERROR_MESSAGE);
            }
            
        }else if(msg[2]==Msg.ESSID_FOUND){            
            final String essid=MsgOptParser.parseESSID(msg, 3, msg.length-3);
            selectbss.setEssid(essid);            
            sb.append(bdle.getString("INF_ESSID"));
            sb.append(' ');
            sb.append(essid);
            final String usermsg=sb.toString();
            MainWindow.updateWindow(
                    new Runnable(){
                        public void run(){
                            window.reportUser(MainWindow.TYPEINFO, usermsg);
                            window.updateESSID(essid);
                        }
                    });
        }else{   
            String msgerr=null;
            if(msg[2]==Msg.CANCELLED){                
                msgerr=bdle.getString("INF_CANC");
                reporter.report(MainWindow.TYPEINFO, msgerr);
                MainWindow.updateWindow(reporter);
            }else if(msg[2]==Msg.NO_RESPONSE){
                msgerr=bdle.getString("ERR_NOAP");
                reporter.report(MainWindow.TYPEERROR, msgerr);
                MainWindow.updateWindow(reporter);
            }else if(msg[2]==Msg.AUTH_ERROR){
                msgerr=bdle.getString("ERR_NOAUTH");
                reporter.report(MainWindow.TYPEERROR, msgerr);
                MainWindow.updateWindow(reporter);
            }else if(msg[2]==Msg.ASSO_ERROR){
                msgerr=bdle.getString("ERR_NOASSO");
                reporter.report(MainWindow.TYPEERROR, msgerr);
                MainWindow.updateWindow(reporter);
            }else                
                System.err.println("Unknown join response: "+
                                   MsgOptParser.reprBytes(msg));
            
            
            final String msginf=bdle.getString("INF_TRY_JOIN");
            MainWindow.updateWindow(
                    new Runnable(){
                        public void run(){
                            window.reportUser(MainWindow.TYPEINFO, msginf);
                            window.enableAccept(true);
                            window.enableCancel(false);
                            window.enableScan(true);
                        }
                    });
        }
    }

    public int getSource(){
        return ((int)Msg.JOINBSS) & 0xFF;
    }

    private void nextStage() throws IllegalStateException{
        Runnable rble=new Runnable(){
                            public void run(){
                                try{
                                    window.nextStage();
                                }catch(IOException io){
                                    io.printStackTrace(System.err);
                                    throw new IllegalStateException(io);
                                }
                            }
                        };

        MainWindow.updateWindow(rble);
    }
}
