// Injection Wizard
// --An application for injecting traffic in WEP protected Wi-Fi networks--
// Version: 1.0
// Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package cliente;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ResourceBundle;


public class ServerDialog extends JDialog{

    public final static int OK=0;
    public final static int BADIP=1;
    public final static int BADPORT=2;

    private int portnum=0;
    private boolean validated=false;
    private ClientConf config=null;
    private ResourceBundle bundle=null;
    private JTextField jTfip=null;
    private JTextField jTfport=null;
    private JButton jBtnok=null;
    private JButton jBtncl=null;

    public ServerDialog(Frame owner, ResourceBundle bundle, ClientConf config){
        super(owner, bundle.getString("MENUSERVER"), true);
        this.bundle=bundle;
        this.config=config;
        boolean hasdefaults=false;
        if(config.getIP()!=null)
            hasdefaults=true;
        
        initComponents(hasdefaults);
        initListeners();
        pack();
        setResizable(false);
        setLocationRelativeTo(owner);
        setVisible(true);
    }

    public boolean isValidated(){
        return validated;
    }

    public String getIP(){
        return jTfip.getText();
    }

    public int getPort(){
        return portnum;
    }

    private void initComponents(boolean setdefaults){
        Dimension dim, d1, d2;
        Container contentPane=getContentPane();
        contentPane.setLayout(new BorderLayout());

        JPanel jPinput=new JPanel(new SpringLayout());
        JLabel label=new JLabel(bundle.getString("LABELSRVIP"));        
        jTfip=new JTextField(15);
        jTfip.setEditable(true);
        jTfip.setToolTipText(bundle.getString("TIPSRVIP"));
        label.setLabelFor(jTfip);
        if(setdefaults)
            jTfip.setText(config.getIP());
        jPinput.add(label);
        jPinput.add(jTfip);

        label=new JLabel(bundle.getString("LABELSRVPORT"));        
        jTfport=new JTextField();
        jTfport.setEditable(true);
        jTfport.setToolTipText(bundle.getString("TIPSRVPORT"));
        jTfport.setMaximumSize(new Dimension(100, 30));
        label.setLabelFor(jTfport);
        if(setdefaults)
            jTfport.setText(Integer.toString(config.getPort()));
        
        jPinput.add(label);
        jPinput.add(jTfport);
        
        SpringUtilities.makeCompactGrid(jPinput, 2, 2, 0, 10, 10, 10);
        jPinput.setBorder(                
                    BorderFactory.createEmptyBorder(10, 20, 10, 10));
        
        contentPane.add(jPinput, BorderLayout.CENTER);
        
        JPanel jPbtns=new JPanel();
        jPbtns.setLayout(new BoxLayout(jPbtns, BoxLayout.LINE_AXIS));
        jPbtns.setBorder(BorderFactory.createEmptyBorder(10, 20, 20, 20));
        jBtnok=new JButton(bundle.getString("BUTTONOK"));
        jBtncl=new JButton(bundle.getString("BUTTONCANCEL"));
        d1=jBtnok.getPreferredSize();
        d2=jBtncl.getPreferredSize();

        if(d1.getWidth()>d2.getWidth())
            dim=d1;
        else
            dim=d2;

        jBtnok.setPreferredSize(dim);
        jBtnok.setMaximumSize(dim);
        jBtncl.setPreferredSize(dim);
        jBtncl.setMaximumSize(dim);

        jPbtns.add(Box.createHorizontalGlue());
        jPbtns.add(jBtnok);
        jPbtns.add(Box.createRigidArea(new Dimension(60, 0)));
        jPbtns.add(Box.createHorizontalGlue());
        jPbtns.add(jBtncl);
        jPbtns.add(Box.createHorizontalGlue());

        contentPane.add(jPbtns, BorderLayout.PAGE_END);
    }
    
    private void initListeners(){
         setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

         jBtnok.addActionListener(
                 new ActionListener(){
                    public void actionPerformed(ActionEvent evt){
                        int val=validateInput();
                        if(val==OK){
                            validated=true;
                            ServerDialog.this.dispose();
                        }else{
                            String msgerr=null;
                            String err=bundle.getString("ERR");
                            if(val==BADIP)
                                msgerr=bundle.getString("ERR_IP");
                            else if(val==BADPORT)
                                msgerr=bundle.getString("ERR_PORT");
                            else
                                msgerr="";

                            JOptionPane.showMessageDialog(ServerDialog.this,
                                                          msgerr, err,
                                                          JOptionPane.ERROR_MESSAGE);
                        }
                    }
         });

         jBtncl.addActionListener(
                 new ActionListener(){
                    public void actionPerformed(ActionEvent evt){
                        ServerDialog.this.dispose();
                    }
         });
     }

    private int validateInput(){
        String input=null;
        int valst=OK;

        input=jTfip.getText();
        if(input==null || input.length()<7)
            valst=BADIP;
        else{
            String[] ipfrags=input.split("\\.");
            int num=0;
            if(ipfrags.length!=4)
                valst=BADIP;
            else{
                for(String frag: ipfrags)
                    try{
                        num=Integer.parseInt(frag);
                        if(num<0 || num>255)
                            throw new NumberFormatException();
                        }catch(NumberFormatException nfe){
                            valst=BADIP;
                        }
            }
        }
        
        if(valst==OK){
            input=jTfport.getText();
            if(input!=null && input.length()>3)
                try{
                    int tmp=Integer.parseInt(input);
                    if(tmp>1023 && tmp<65536){
                        portnum=tmp;
                    }
                    else
                        valst=BADPORT;
                }catch(NumberFormatException nfe){
                    valst=BADPORT;
                }
            else
                valst=BADPORT;
        }
        
        return valst;
    }
}
