// Injection Wizard
// --An application for injecting traffic in WEP protected Wi-Fi networks--
// Version: 1.0
// Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package cliente;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ResourceBundle;
import java.util.HashMap;

public class SubnetDialog extends JDialog{

    private final static int OK=0;
    private final static int BADIP=-2;
    private final static int BADTYPE=-1;
    private final static int NOPRIVATEIP=-3;

    private boolean validated=false;
    private ResourceBundle bundle=null;
    private ClientConf config=null;
    private JTextField jTfaddr1=null;
    private JTextField jTfaddr2=null;
    private ButtonGroup bgsearch=null;
    private JButton jBtnok=null;
    private JButton jBtncl=null;
    private HashMap<ButtonModel, Integer> searchvalues=null;

    public SubnetDialog(Frame owner, ResourceBundle bundle, ClientConf config){
        super(owner, bundle.getString("MENUSUBNET"), true);
        this.bundle=bundle;
        this.config=config;
        initComponents();
        initListeners();
        pack();
        setResizable(false);
        setLocationRelativeTo(owner);
        setVisible(true);
    }

    public boolean isValidated(){
        return validated;
    }

    public int getSearchType(){
        ButtonModel bm=bgsearch.getSelection();
        Integer val=searchvalues.get(bm);
        return val.intValue();
    }

    public String getSubnetPrefix(){
        StringBuffer sb=new StringBuffer(7);
        sb.append(jTfaddr1.getText());
        sb.append('.');
        sb.append(jTfaddr2.getText());

        return sb.toString();
    }

    private void initComponents(){
        String label=null;
        Container contentPane=getContentPane();
        contentPane.setLayout(new BorderLayout());
        
        JPanel jPcenter=new JPanel(new GridLayout(2,1));
        JPanel jPup=new JPanel();
        JPanel jPdown=new JPanel(new GridLayout(1,2));
        JPanel jPbtns=new JPanel();

        label=bundle.getString("LABELNET");
        jPup.setBorder(BorderFactory.createCompoundBorder(
                            BorderFactory.createTitledBorder(label),
                            BorderFactory.createEmptyBorder(10, 20, 10, 10)));
        
        jPup.setLayout(new BoxLayout(jPup, BoxLayout.LINE_AXIS));
        label=bundle.getString("TIPNET");
        IPprefixVerifier ipverif=new IPprefixVerifier();
        jTfaddr1=new JTextField(3);
        jTfaddr1.setToolTipText(label);
        jTfaddr1.setInputVerifier(ipverif);
        jTfaddr2=new JTextField(3);
        jTfaddr2.setToolTipText(label);
        jTfaddr2.setInputVerifier(ipverif);
        JLabel dotlabel=new JLabel("<html><font size=+1><b>.</b></font>");
        dotlabel.setHorizontalAlignment(SwingConstants.CENTER);
        dotlabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        String iprefix=config.getSubnet();
        if(iprefix!=null){
            String[] ipfrags=iprefix.split("\\.");
            jTfaddr1.setText(ipfrags[0]);
            jTfaddr2.setText(ipfrags[1]);
        }
        jPup.add(jTfaddr1);
        jPup.add(dotlabel);
        jPup.add(jTfaddr2);
        jPcenter.add(jPup);

        label=bundle.getString("LABELSEARCH");
        jPdown.setBorder(BorderFactory.createCompoundBorder(
                            BorderFactory.createTitledBorder(label),
                            BorderFactory.createEmptyBorder(10, 20, 10, 20)));
        bgsearch=new ButtonGroup();        
        searchvalues=new HashMap<ButtonModel, Integer>(2);
        JRadioButton jRbtn=null;
        int selectype=config.getSearchType();

        label=bundle.getString("LABELSEARCH1");
        jRbtn=new JRadioButton(label, selectype==0);
        label=bundle.getString("TIPSEARCH1");
        jRbtn.setToolTipText(label);
        bgsearch.add(jRbtn);
        searchvalues.put(jRbtn.getModel(), new Integer(0));
        jPdown.add(jRbtn);

        label=bundle.getString("LABELSEARCH2");
        jRbtn=new JRadioButton(label, selectype==1);
        label=bundle.getString("TIPSEARCH2");
        jRbtn.setToolTipText(label);
        bgsearch.add(jRbtn);
        searchvalues.put(jRbtn.getModel(), new Integer(1));
        jPdown.add(jRbtn);
        jPcenter.add(jPdown);
        contentPane.add(jPcenter, BorderLayout.CENTER);

        jPbtns.setLayout(new BoxLayout(jPbtns, BoxLayout.LINE_AXIS));
        jPbtns.setBorder(BorderFactory.createEmptyBorder(20, 30, 20, 30));
        label=bundle.getString("BUTTONOK");
        jBtnok=new JButton(label);
        Dimension dim1=jBtnok.getPreferredSize();
        label=bundle.getString("BUTTONCANCEL");
        jBtncl=new JButton(label);
        Dimension dim2=jBtncl.getPreferredSize();

        if(dim1.getWidth()>dim2.getWidth()){
            jBtncl.setPreferredSize(dim1);
            jBtnok.setMaximumSize(dim1);
        }else{
            jBtnok.setPreferredSize(dim2);
            jBtncl.setMaximumSize(dim2);
        }

        Dimension dim=new Dimension(50, 0);
        jPbtns.add(Box.createHorizontalGlue());
        jPbtns.add(jBtnok);
        jPbtns.add(Box.createRigidArea(dim));
        jPbtns.add(jBtncl);
        jPbtns.add(Box.createHorizontalGlue());
        contentPane.add(jPbtns, BorderLayout.SOUTH);
    }

    private void initListeners(){
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        jBtnok.addActionListener(
                new ActionListener(){
                    public void actionPerformed(ActionEvent evt){
                        String errmsg=bundle.getString("ERR_SELECT");
                        String err=bundle.getString("ERR");
                        int valcode=OK;

                        valcode=validateInput();
                        if(valcode==BADTYPE)
                            errmsg+=" \""+bundle.getString("LABELSEARCH")+"\"";
                        else if(valcode==BADIP)
                            errmsg=bundle.getString("ERR_SUBNET");
                        else if(valcode==NOPRIVATEIP)
                            errmsg=bundle.getString("ERR_IPRIV");
                        else if(valcode==OK)
                            errmsg=null;

                        if(errmsg==null){
                            validated=true;
                            dispose();
                        }
                        else
                            JOptionPane.showMessageDialog(
                                            SubnetDialog.this, errmsg, err,
                                            JOptionPane.ERROR_MESSAGE);
                    }
            });
            
        jBtncl.addActionListener(
                new ActionListener(){
                    public void actionPerformed(ActionEvent evt){
                        dispose();
                    }            
                });
    }

    private int validateInput(){
        if(bgsearch.getSelection()==null)
            return BADTYPE;
        else
            try{
                int fstbyte=Integer.parseInt(jTfaddr1.getText());
                int sndbyte=Integer.parseInt(jTfaddr2.getText());
                boolean valid=false;
                valid=(fstbyte==10) ||
                      (fstbyte==172 && sndbyte>15 && sndbyte<32) ||
                      (fstbyte==169 && sndbyte==254) ||
                      (fstbyte==192 && sndbyte==168);
                if(valid)
                    return OK;
                else
                    return NOPRIVATEIP;
            }catch(NumberFormatException nfe){
                return BADIP;
            }
    }
}



class IPprefixVerifier extends InputVerifier{

    public boolean verify(JComponent control){
        JTextField field=(JTextField) control;
        String text=field.getText();
        boolean validated=false;

        try{
            int val=Integer.parseInt(text);
            if(val>-1 && val<256)
                validated=true;
        }catch(NumberFormatException nf){}

        return validated;
    }
    
    @Override
    public boolean shouldYieldFocus(JComponent control){
        if(!verify(control)){
            JTextField field=(JTextField)control;
            field.setText("");
        }
        return true;
    }
}