# -*- coding: UTF-8 -*-
# Injection Wizard
# --An application for injecting traffic in WEP protected Wi-Fi networks--
# Version: 1.0
# Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

from scapy.packet import *
from scapy.layers.dot11 import *
from scapy.layers.l2 import ARP

def frame_to_station(pkt, stamac):  
    
    if pkt.haslayer(Dot11):
        dot11=pkt.getlayer(Dot11)
        return dot11.getfieldval('addr1')==stamac
    
    return False

def is_Data_frame_to_client(pkt, cliMAC):    

    if pkt.haslayer(Dot11):
        dot11=pkt.getlayer(Dot11)
        if dot11.getfieldval('type')==2 and dot11.getfieldval('subtype')==0:
            return ((dot11.getfieldval('FCfield') & 0x02) !=0) and\
                    dot11.getfieldval('addr1')==cliMAC
                    
    return False

def is_Data_frame_from_client(pkt, cliMAC):

    if pkt.haslayer(Dot11):
        dot11=pkt.getlayer(Dot11)
        if dot11.getfieldval('type')==2 and dot11.getfieldval('subtype')==0:
            return ((dot11.getfieldval('FCfield') & 0x01) !=0) and\
                    dot11.getfieldval('addr2')==cliMAC

    return False

def is_ARP_client(pkt, cliMAC):    

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.getlayer(ARP)):
            return dot11.getfieldval('FCfield')&0x01==1 and\
                   dot11.getfieldval('addr2')==cliMAC

    return False

def is_ARP_client_relayed(pkt, cliMAC):    

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.getlayer(ARP)):
            return dot11.getfieldval('FCfield')&0x02==2 and\
                   dot11.getfieldval('addr3')==cliMAC

    return False


def is_WEP_into_BSS(pkt, bssid):    

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.getfieldval('type')==2 and dot11.haslayer(Dot11WEP)):
            flags=dot11.getfieldval('FCfield')
            return (flags & 0x02 != 0 and dot11.getfieldval('addr2')==bssid) or\
                   (flags & 0x01 != 0 and dot11.getfieldval('addr1')==bssid)
                   
    return False

def is_Data_client_relayed(pkt, cliMAC):
    
    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.getfieldval('type')==2):
            return dot11.getfieldval('FCfield')&0x02==2 and\
                   dot11.getfieldval('addr3')==cliMAC                   
    
    return False


def is_ACK(pkt, cliMAC):    

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        return dot11.getfieldval('type')==1 and\
               dot11.getfieldval('subtype')==13 and\
               dot11.getfieldval('addr1')==cliMAC

    return False

def is_Probe_Response(pkt, cliMAC):    

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        return dot11.getfieldval('type')==0 and\
               dot11.getfieldval('subtype')==5 and\
               dot11.getfieldval('addr1')==cliMAC

    return False

def is_Probe_Response_from_AP(pkt, bssid):    

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        return dot11.getfieldval('type')==0 and\
               dot11.getfieldval('subtype')==5 and\
               dot11.getfieldval('addr3')==bssid
               
    return False

def is_Deauthentication_from_AP(pkt, climac, apmac):   
    
    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        return dot11.getfieldval('type')==0 and\
               dot11.getfieldval('subtype')==12 and\
               dot11.getfieldval('addr1')==climac and\
               dot11.getfieldval('addr2')==apmac and\
               dot11.haslayer(Dot11Deauth)
            
    return False

def is_Disassociation_from_AP(pkt, climac, apmac):
    
    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        return dot11.getfieldval('type')==0 and\
               dot11.getfieldval('subtype')==10 and\
               dot11.getfieldval('addr1')==climac and\
               dot11.getfieldval('addr2')==apmac and\
               dot11.haslayer(Dot11Disas)
            
    return False

def is_Null_Data_client(pkt, cliMAC):
    
    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        return dot11.getfieldval('type')==2 and\
               dot11.getfieldval('subtype')==4 and\
               dot11.getfieldval('FCfield')&0x01==1 and\
               dot11.getfieldval('addr2')==cliMAC

    return False

def is_OSA_Req(pkt, cliMAC):
   
    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.haslayer(Dot11Auth)):
            auth=dot11.getlayer(Dot11Auth)
            return dot11.getfieldval('addr2')==cliMAC and\
                   auth.getfieldval('algo')==0 and\
                   auth.getfieldval('seqnum')==1

    return False

def is_OSA_Resp(pkt, cliMAC):   

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.haslayer(Dot11Auth)):
            auth=dot11.getlayer(Dot11Auth)
            return dot11.getfieldval('addr1')==cliMAC and\
                   auth.getfieldval('algo')==0 and\
                   auth.getfieldval('seqnum')==2

    return False


def is_Assoc_Req(pkt, cliMAC):    

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.haslayer(Dot11AssoReq)):
            return dot11.getfieldval('addr2')==cliMAC

    return False


def is_Assoc_Req_to_AP(pkt, bssid):    

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.haslayer(Dot11AssoReq)):
            return dot11.getfieldval('addr3')==bssid

    return False


def is_Assoc_Resp(pkt, cliMAC):   

    if(pkt.haslayer(Dot11)):
        dot11=pkt.getlayer(Dot11)
        if(dot11.haslayer(Dot11AssoResp)):            
            return dot11.getfieldval('addr1')==cliMAC

    return False


