# -*- coding: UTF-8 -*-
# Injection Wizard
# --An application for injecting traffic in WEP protected Wi-Fi networks--
# Version: 1.0
# Copyright (C) 2010 Fernando Pablo Romero Navarro <fernpromero@gmail.com>
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import sys
import time
import socket
import logging
import threading

import msg
import control
import logger
from scapy.layers import dot11


class WorkerThread(threading.Thread):
    

    DEFAULT_TIMEOUT=120.0

    def __init__(self, _function, _args=None):
        threading.Thread.__init__(self)
        self.answer=None
        self.function=_function
        self.answeready=threading.Event()

        if _args is None:
            self.args=[self]
        else:
            self.args=_args
            self.args.append(self)
            

    def run(self):
        global workerthread, thread_at_work
       
        control.cancel.clear_cancelled()
        
        try:               
            self.function(*self.args)
        except:     
            raise
        finally:
            workerthread=None            
            thread_at_work=False
            control.cancel.clear_cancelled()

    def wakeup(self):
        self.answeready.set()
            

    def client_response(self, option):        
        self.answer=option
        self.answeready.set()

    def ask_client(self, funid, qstid, options, default, timeout=None, **kargs):        
        
        if timeout is None:
            timeout=self.DEFAULT_TIMEOUT

        response=msg.ASK+funid+qstid
        for opt in options:
            response=response+opt

        response=response+msg.ITEM_SEP

        if kargs is not None and len(kargs)>0:
            for entry in kargs.iteritems():
                response=response+entry[0]+msg.VALUE_SEP+entry[1]+msg.ITEM_SEP

        self.answer=default
        self.answeready.clear()
                
        try:
            commsrv=control.get_lock_channel()
            commsrv.writemsg(response)
            control.release_channel()
        except:
            logger.srvlogger.exception("Connection with client is broken")

        self.answeready.wait(timeout)
        return self.answer


def usage():
    print "Usage: sudo python main.py <network interface IP address> <port>"

def exiterror(code):

    if(code==-1):
        print "IP address is not valid"
    elif(code==-2):
        print "Port number is not valid"

    exit(code)

def validate_args(ip, port):
    
    try:
        ip_ok=False
        tmp=ip.strip()
        values=tmp.split(".")
        inrange=lambda x: x>=0 and x<=255
        values=map(int, values)
        values=map(inrange, values)
        forall=lambda x,y: x and y

        if reduce(forall, values, True):
            ip_ok=True
        else:
            return -1

        portnum=int(port)
        if(portnum>1000 and portnum<65536):
            return 0
        else:
            return -2

    except ValueError:
        if(ip_ok):
            return -2
        else:
            return -1

def init():

    logger.init_server_logger()

def client_hello():
    global thread_at_work

    control.get_config(msg.GET+msg.CONFIG, None, thread_at_work)

def get_bss_info():
    global thread_at_work

    control.get_config(msg.GET+msg.JOINEDBSS, None, thread_at_work)
    

def request_cancel():
    global thread_at_work, workerthread
    respmsg=None
    commsrv=None

    if thread_at_work:
        control.cancel.set_cancelled()
        workerthread.wakeup()
        respmsg=msg.RESP+msg.DOING
    else:
        respmsg=msg.RESP+msg.NOT_APPLICABLE

    try:
        commsrv=control.get_lock_channel()        
        commsrv.writemsg(respmsg)
        control.release_channel()
    except Exception:
        control.release_channel()
        raise

def disconnect():
    
    respmsg=msg.RESP+msg.OK
    
    try:
        commsrv=control.get_lock_channel()        
        commsrv.writemsg(respmsg)
        control.release_channel()
        control.change_offline()
    except Exception:
        control.release_channel()
        raise


def ready_to_exit():   
    global thread_at_work
    waitime=0.5
    respmsg=msg.RESP+msg.OK


    try:
        commsrv=control.get_lock_channel()        
        commsrv.writemsg(respmsg)
        control.release_channel()
        control.change_offline()
    except Exception:
        control.release_channel()
        logger.srvlogger.exception("Error while exiting")

    if thread_at_work:
        control.cancel.set_cancelled()
        while thread_at_work:
            time.sleep(waitime)


SLEEP_TIME=0.3

if __name__ == "__main__":
    
    thread_at_work=False
    workerthread=None
   
    argc=len(sys.argv)
    if(argc<3):
        usage()
        sys.exit(-1)

    ip=sys.argv[1]
    port=sys.argv[2]
    code=validate_args(ip, port)
    if(code!=0):
        exiterror(code)
    
    init()
    print " INFO - Server initialization done"
    srvsock=None
    connsock=None    
    alive=True
    connected=False

    try:
        srvsock=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        srvsock.bind((ip, int(port)))
        srvsock.listen(1)
        print " INFO - Server awaiting connections at IP address/port: %s/%s"\
                %(ip, port)
                
        climsg=None

        while(alive):
            connsock, addr=srvsock.accept()
            control.change_online(connsock)
            connected=True
            print " INFO - Connection received from IP address/source port: %s/%s"\
                    %addr

            while(connected):
                try:                    
                    climsg=control.comserv.readmsg()
                    cmd=climsg[0]                   
                    
                    if cmd==msg.EXIT:
                        connected=False
                        alive=False
                        ready_to_exit()
                    elif cmd==msg.DISCONNECT:
                        connected=False
                        disconnect()
                    elif cmd==msg.CANCEL:
                        request_cancel()
                    elif cmd==msg.HELLO:
                        client_hello()
                    elif cmd==msg.GETBSS:
                        get_bss_info()
                    elif cmd==msg.ANSWER:
                        answdest=workerthread
                        if answdest is not None:
                            answdest.client_response(climsg[1:])
                    elif thread_at_work:
                        logger.srvlogger.exception("There is a task running yet")
                    elif cmd==msg.GET:
                        workerthread=WorkerThread(control.get_config, [climsg])
                        thread_at_work=True
                        workerthread.start()                    
                    elif cmd==msg.SET:
                        workerthread=WorkerThread(control.set_config, [climsg])
                        thread_at_work=True
                        workerthread.start()
                    elif cmd==msg.SCAN:
                        workerthread=WorkerThread(control.process_scan, [])
                        thread_at_work=True
                        workerthread.start()
                    elif cmd==msg.JOIN:
                        workerthread=WorkerThread(control.join_bss, [climsg])
                        thread_at_work=True
                        workerthread.start()
                    elif cmd==msg.CAPTURE:
                        workerthread=WorkerThread(control.capture, [])
                        thread_at_work=True
                        workerthread.start()                    
                    elif cmd==msg.EXTEND:
                        workerthread=WorkerThread(control.extend, [])
                        thread_at_work=True
                        workerthread.start()
                    elif cmd==msg.FIND:
                        workerthread=WorkerThread(control.find_host, [])
                        thread_at_work=True
                        workerthread.start()
                    elif cmd==msg.INJECT:
                        workerthread=WorkerThread(control.inject, [])
                        thread_at_work=True
                        workerthread.start()
                    else:
                        pass

                except socket.timeout:
                    time.sleep(SLEEP_TIME)
                except socket.error:
                    logger.srvlogger.exception("Connection is lost")
                    try:
                        control.change_offline()
                    except:
                        pass
                    finally:
                        connected=False

        
        print " INFO - Stopping the server..."
        srvsock.close()        
        
    except socket.error:
        logger.srvlogger.exception("Error while opening or closing a connection")
    finally:
        logging.shutdown()
